@@ -230,64 +230,6 @@ impl<K, V, S> LinkedHashMap<K, V, S> {
         }
     }
 
-    #[cfg_attr(feature = "inline-more", inline)]
-    pub fn retain<F>(&mut self, mut f: F)
-    where
-        F: FnMut(&K, &mut V) -> bool,
-    {
-        // We do not drop the key and value when a value is filtered from the map during the call to
-        // `retain`.  We need to be very careful not to have a live `HashMap` entry pointing to
-        // either a dangling `Node` or a `Node` with dropped keys / values.  Since the key and value
-        // types may panic on drop, they may short-circuit the entry in the map actually being
-        // removed.  Instead, we push the removed nodes onto the free list eagerly, then try and
-        // drop the keys and values for any newly freed nodes *after* `HashMap::retain` has
-        // completely finished.
-        struct DropFilteredValues<'a, K, V> {
-            free: &'a mut Option<NonNull<Node<K, V>>>,
-            cur_free: Option<NonNull<Node<K, V>>>,
-        }
-
-        impl<'a, K, V> DropFilteredValues<'a, K, V> {
-            #[cfg_attr(feature = "inline-more", inline)]
-            fn drop_later(&mut self, node: NonNull<Node<K, V>>) {
-                unsafe {
-                    detach_node(node);
-                    push_free(&mut self.cur_free, node);
-                }
-            }
-        }
-
-        impl<'a, K, V> Drop for DropFilteredValues<'a, K, V> {
-            fn drop(&mut self) {
-                unsafe {
-                    let end_free = self.cur_free;
-                    while self.cur_free != *self.free {
-                        let cur_free = self.cur_free.as_ptr();
-                        (*cur_free).take_entry();
-                        self.cur_free = (*cur_free).links.free.next;
-                    }
-                    *self.free = end_free;
-                }
-            }
-        }
-
-        let free = self.free;
-        let mut drop_filtered_values = DropFilteredValues {
-            free: &mut self.free,
-            cur_free: free,
-        };
-
-        self.map.retain(|&node, _| unsafe {
-            let (k, v) = (*node.as_ptr()).entry_mut();
-            if f(k, v) {
-                true
-            } else {
-                drop_filtered_values.drop_later(node);
-                false
-            }
-        });
-    }
-
     #[cfg_attr(feature = "inline-more", inline)]
     pub fn hasher(&self) -> &S {
         &self.hash_builder
@@ -357,6 +299,21 @@ where
         }
     }
 
+    #[cfg_attr(feature = "inline-more", inline)]
+    pub fn get_refresh<Q>(&mut self, k: &Q) -> Option<&mut V>
+    where
+        K: Borrow<Q>,
+        Q: Hash + Eq + ?Sized,
+    {
+        match self.raw_entry_mut().from_key(k) {
+            RawEntryMut::Occupied(mut occupied) => {
+                occupied.to_back();
+                Some(occupied.into_mut())
+            }
+            RawEntryMut::Vacant(_) => None,
+        }
+    }
+
     /// Inserts the given key / value pair at the *back* of the internal linked list.
     ///
     /// Returns the previously set value, if one existed prior to this call.  After this call,
@@ -396,7 +353,7 @@ where
         K: Borrow<Q>,
         Q: Hash + Eq + ?Sized,
     {
-        match self.raw_entry_mut().from_key(&k) {
+        match self.raw_entry_mut().from_key(k) {
             RawEntryMut::Occupied(occupied) => Some(occupied.remove()),
             RawEntryMut::Vacant(_) => None,
         }
@@ -408,12 +365,85 @@ where
         K: Borrow<Q>,
         Q: Hash + Eq + ?Sized,
     {
-        match self.raw_entry_mut().from_key(&k) {
+        match self.raw_entry_mut().from_key(k) {
             RawEntryMut::Occupied(occupied) => Some(occupied.remove_entry()),
             RawEntryMut::Vacant(_) => None,
         }
     }
 
+    #[cfg_attr(feature = "inline-more", inline)]
+    pub fn retain<F>(&mut self, mut f: F)
+    where
+        F: FnMut(&K, &mut V) -> bool,
+    {
+        // We do not drop the key and value when a value is filtered from the map during the call to
+        // `retain`.  We need to be very careful not to have a live `HashMap` entry pointing to
+        // either a dangling `Node` or a `Node` with dropped keys / values.  Since the key and value
+        // types may panic on drop, they may short-circuit the entry in the map actually being
+        // removed.  Instead, we push the removed nodes onto the free list eagerly, then try and
+        // drop the keys and values for any newly freed nodes *after* `HashMap::retain` has
+        // completely finished.
+        struct DropFilteredValues<'a, K, V> {
+            free: &'a mut Option<NonNull<Node<K, V>>>,
+            cur_free: Option<NonNull<Node<K, V>>>,
+        }
+
+        impl<'a, K, V> DropFilteredValues<'a, K, V> {
+            #[inline]
+            fn drop_later(&mut self, node: NonNull<Node<K, V>>) {
+                unsafe {
+                    detach_node(node);
+                    push_free(&mut self.cur_free, node);
+                }
+            }
+        }
+
+        impl<'a, K, V> Drop for DropFilteredValues<'a, K, V> {
+            fn drop(&mut self) {
+                unsafe {
+                    let end_free = self.cur_free;
+                    while self.cur_free != *self.free {
+                        let cur_free = self.cur_free.as_ptr();
+                        (*cur_free).take_entry();
+                        self.cur_free = (*cur_free).links.free.next;
+                    }
+                    *self.free = end_free;
+                }
+            }
+        }
+
+        let free = self.free;
+        let mut drop_filtered_values = DropFilteredValues {
+            free: &mut self.free,
+            cur_free: free,
+        };
+
+        if let Some(values) = self.values {
+            unsafe {
+                let mut cur = values.as_ref().links.value.next;
+                while cur != values {
+                    let next = cur.as_ref().links.value.next;
+                    let (k, v) = (*cur.as_ptr()).entry_mut();
+                    if !f(k, v) {
+                        let hash = hash_key(&self.hash_builder, k);
+                        match self
+                            .map
+                            .raw_entry_mut()
+                            .from_hash(hash, |o| (*o).as_ref().key_ref().eq(k))
+                        {
+                            hash_map::RawEntryMut::Occupied(entry) => {
+                                entry.remove();
+                                drop_filtered_values.drop_later(cur);
+                            }
+                            hash_map::RawEntryMut::Vacant(_) => unreachable!(),
+                        }
+                    }
+                    cur = next;
+                }
+            }
+        }
+    }
+
     #[cfg_attr(feature = "inline-more", inline)]
     pub fn pop_front(&mut self) -> Option<(K, V)> {
         if self.is_empty() {
